@@ -1,9 +1,9 @@
-# Copyright 2001-2006 The Apache Software Foundation or its licensors, as
-# applicable.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
@@ -74,6 +74,7 @@ sub run_t {
         my $top_dir = Apache::Test::vars('top_dir');
 
         foreach my $lib (catfile($top_dir, qw(Apache-Test lib)),
+                         catfile($top_dir, qw(.. Apache-Test lib)),
                          catfile($top_dir, 'lib')) {
 
             if (-d $lib) {
@@ -86,7 +87,7 @@ sub run_t {
             }
         }
     }
-    
+
     my $cmd = qq[$^X $source_lib $file];
 
     my $h = Symbol::gensym();
@@ -140,15 +141,15 @@ sub get_tests {
     my $ts = $args->{tests} || [];
 
     if (@$ts) {
-	for (@$ts) {
-	    if (-d $_) {
-		push(@tests, sort <$base/$_/*.t>);
-	    }
-	    else {
-		$_ .= ".t" unless /\.t$/;
-		push(@tests, $_);
-	    }
-	}
+        for (@$ts) {
+            if (-d $_) {
+                push(@tests, sort <$base/$_/*.t>);
+            }
+            else {
+                $_ .= ".t" unless /\.t$/;
+                push(@tests, $_);
+            }
+        }
     }
     else {
         if ($args->{tdirs}) {
